﻿Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Threading.Tasks
Imports Dataq.Devices
Imports Dataq.Devices.DI2108P
Imports Dataq.Platforms.Windows


Public Class Form1
    Dim Synced As Dataq.Devices.DI2108P.Synchronized
    Dim SyncedInstrumentPool As List(Of Dataq.Devices.DI2108P.Device)
    Dim TaskRead As Task
    Dim CancelRead As CancellationTokenSource
    ' Define class that will print the data to the GUI
    Private progress As IProgress(Of String) =
        New Progress(Of String)(Sub(s As String)
                                    tbOutput.AppendText(s)
                                End Sub)

    Private Async Sub btnDiscover_Click(sender As Object, e As EventArgs) Handles btnDiscover.Click

        Dim DiscoveredDI2108Ps As IReadOnlyList(Of Dataq.Devices.IDevice)
        DiscoveredDI2108Ps = Await Dataq.Misc.Discovery.ByModel(GetType(Dataq.Devices.DI2108P.Device))
        If DiscoveredDI2108Ps.Count > 1 Then
            tbOutput.AppendText("Found multiple DI-2108-Ps:" + vbCrLf)
            ' Group desired devices to be synchronized
            Synced = New Dataq.Devices.DI2108P.Synchronized(DiscoveredDI2108Ps)
            ' Ensure they're stopped
            Await Synced.AcquisitionStopAsync()
            ' Cast devices to specific DI-2108-P for more detail
            SyncedInstrumentPool = New List(Of Dataq.Devices.DI2108P.Device)(DiscoveredDI2108Ps.Cast(Of Dataq.Devices.DI2108P.Device))
            ' Query devices for some info
            For Each Instrument In SyncedInstrumentPool
                Await Instrument.QueryDeviceAsync()
                ' Print queried info to GUI
                tbOutput.AppendText(Instrument.Manufacturer + " " + Instrument.Model + " (rev. " +
                                      Instrument.FirmwareRevision.ToString() + "): " +
                                      Instrument.Serial + vbCrLf)
            Next
            'discovery is finished. Enable Setup button
            btnDiscover.Enabled = False
            btnSetup.Enabled = True
        Else
            tbOutput.AppendText("Multiple DI-2108-P were not found. Ensure that instruments " +
                                "are connected to the same USB hub." + vbCrLf + vbCrLf)
        End If
    End Sub

    Private Async Sub btnSetup_Click(sender As Object, e As EventArgs) Handles btnSetup.Click
        Dim AnalogChan As Dataq.Devices.DI2108P.AnalogVoltageIn
        Dim DigitalChan As Dataq.Devices.DI2108P.DigitalIn
        'Configure sample rate for a modestly slow rate of 10 Hz to keep the display tame:
        '4000 Hz / 2 channels / 200 sample decimation factor
        Dim Decimation As Integer = 200
        Dim DeviceSampleRate As Double = 4000

        'Configure each instrument in the available pool for:
        'Analog channel 1, ±10 v range, Average acquisition mode
        'Digital input port 0
        For Each Instrument In SyncedInstrumentPool
            ' Clear/Remove all channels from device
            Instrument.Channels.Clear()
            ' Add/Enable the analog channel (physical channel #1)
            ' Cast it from the generic channel returned, to the specific DI-2108-P analog channel type
            AnalogChan = CType(Instrument.ChannelFactory(1, GetType(Dataq.Devices.DI2108P.AnalogVoltageIn)),
                Dataq.Devices.DI2108P.AnalogVoltageIn)
            AnalogChan.InputRange.Maximum = 10
            AnalogChan.InputRange.Minimum = -10
            AnalogChan.AcquisitionMode = New Dataq.Devices.DI2108P.AverageSample
            AnalogChan.AcquisitionMode.Samples = Decimation

            'Add digital port 0 
            DigitalChan = CType(Instrument.ChannelFactory(0, GetType(Dataq.Devices.DI2108P.DigitalIn)),
             Dataq.Devices.DI2108P.DigitalIn)
            DigitalChan.AcquisitionMode.Samples = Decimation

            'set the device sampling rate
            Instrument.SampleRate = DeviceSampleRate
        Next
        ' Initialize devices with the configuration
        Await Synced.InitializeAsync()

        'the Class attempts to set an optimal packet size based upon sampling rate, 
        'decimation factor, and the number of enabled channels. Since this example
        'defines such a slow sampling rate of 10 Hz, force the instruments to use 
        'the smallest packet size to keep the display of acquired data fluid. Note
        'that this must be done AFTER Synced.InitializeAsync(), since that instruction
        'sets the selected optimal packet size. 
        For Each instrument In SyncedInstrumentPool
            Await instrument.Protocol.SetPacketSizeAsync(16)
        Next

        ' Display device and channel information
        Dim DeviceNumber As Integer = 0
        For Each Instrument In SyncedInstrumentPool
            tbOutput.AppendText("Instrument number: " + DeviceNumber.ToString + Environment.NewLine)
            DeviceNumber = DeviceNumber + 1
            tbOutput.AppendText("      Instrument sample rate: " + Instrument.SampleRate.ToString() + Environment.NewLine)
            For Each el As Dataq.Devices.DI2108P.ChannelIn In Instrument.Channels
                tbOutput.AppendText("      Channel Type: " + el.GetType().ToString() + Environment.NewLine)
                tbOutput.AppendText("      Channel Number: " + el.Number.ToString() + Environment.NewLine)
                tbOutput.AppendText("      Sample Rate: " + el.GetSampleRate(Instrument).ToString() + Environment.NewLine + vbCrLf)
            Next
        Next
        btnSetup.Enabled = False
        btnState.Enabled = True
    End Sub

    Private Async Sub btnState_Click(sender As Object, e As EventArgs) Handles btnState.Click
        Dim len As Integer ' the number of values in the DataIn buffer
        Dim temp As String = "" 'GUI data output strings

        ' Disable button to avoid extra clicks
        btnState.Enabled = False
        ' Cancel read operation
        If CancelRead IsNot Nothing Then
            CancelRead.Cancel()
            CancelRead = Nothing
        End If
        ' Wait for canceled read to complete
        If TaskRead IsNot Nothing Then
            Await TaskRead
            TaskRead = Nothing
        End If
        ' Check current state
        If Synced.IsAcquiring Then
            ' Currently acquiring, so stop it
            Await Synced.AcquisitionStopAsync()
            ' Change button text/state
            btnState.Text = "Acquire"
        Else
            'get here if not acquiring, so must be starting an acquisition session
            ' Create cancelation token
            CancelRead = New CancellationTokenSource()
            ' NOTE: assuming all channels are input channels
            ' Clear channel data
            For Each Instrument In SyncedInstrumentPool
                For Each Channel In Instrument.Channels
                    CType(Channel, Dataq.Devices.IChannelIn).DataIn.Clear()
                Next
            Next
            ' Start acquisition
            Await Synced.AcquisitionStartAsync(False)
            ' Change button text
            btnState.Text = "Stop"
            ' Start a task in the background to read data
            TaskRead = Task.Run(Async Function()
                                    ' Keep reading while acquiring data
                                    While Synced.IsAcquiring
                                        ' Read data and catch to exit loop if canceled
                                        Try
                                            Await Synced.ReadDataAsync(CancelRead.Token)
                                        Catch ex As AggregateException
                                            If ex.InnerException.GetType() Is GetType(OperationCanceledException) Then
                                                Exit While
                                            End If
                                        End Try
                                        If CType(SyncedInstrumentPool(0).Channels(0), Dataq.Devices.IChannelIn).DataIn.Count = 0 Then
                                            'get here if no data in the input buffer...continue
                                            Continue While
                                        End If
                                        'We have data, so...
                                        ' Convert data to strings
                                        'Data is output in this format:
                                        'Device(0)              Device(1)           ...    Device(n)
                                        'analog1,digital0       analog1,digital0    ...    analog1,digital0
                                        'analog1,digital0       analog1,digital0    ...    analog1,digital0
                                        '...                    ...                        ...
                                        'analog1,digital0       analog1,digital0    ...    analog1,digital0
                                        temp = Nothing
                                        'determine the number of values in DataIn buffer
                                        len = CType(SyncedInstrumentPool(0).Channels(0), Dataq.Devices.IChannelIn).DataIn.Count - 1
                                        ' NOTE: assuming all channels are input channels
                                        ' NOTE: assuming all channels contain exact same amount of data
                                        For index = 0 To len
                                            For Each Instrument In SyncedInstrumentPool
                                                For Each Channel In Instrument.Channels
                                                    If (TypeOf Channel Is Dataq.Devices.IAnalogVoltage) Then
                                                        'do some formatting out the output strings...real if analog in; integer if digital in
                                                        temp += " " + CType(Channel, Dataq.Devices.IChannelIn).DataIn(index).ToString("0.000") + ","
                                                    Else
                                                        temp += " " + CType(Channel, Dataq.Devices.IChannelIn).DataIn(index).ToString("0")
                                                    End If
                                                Next
                                                temp += "      "
                                            Next
                                            temp += Environment.NewLine
                                        Next
                                        ' NOTE: assuming all channels are input channels
                                        ' Clear "used" data
                                        For Each Instrument In SyncedInstrumentPool
                                            For Each ch In Instrument.Channels
                                                CType(ch, Dataq.Devices.IChannelIn).DataIn.Clear()
                                            Next
                                        Next
                                        ' Report the data to be printed to the GUI
                                        progress.Report(temp)
                                    End While
                                    progress.Report("Stopped" + Environment.NewLine)
                                End Function, CancelRead.Token)
        End If
        ' Enable button once acquisition has completed
        btnState.Enabled = True
    End Sub
End Class
